import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";

/**
 * The `DragRotateHandler` allows the user to rotate the map by clicking and
 * dragging the cursor while holding the right mouse button or `ctrl` key.
 */
var DragRotateHandler = /*#__PURE__*/function () {
  /**
   * @param {Object} [options]
   * @param {number} [options.bearingSnap] The threshold, measured in degrees, that determines when the map's
   *   bearing will snap to north.
   * @param {bool} [options.pitchWithRotate=true] Control the map pitch in addition to the bearing
   * @private
   */
  function DragRotateHandler(options, mouseRotate, mousePitch) {
    _classCallCheck(this, DragRotateHandler);

    this.pitchWithRotate = options.pitchWithRotate;
    this.mouseRotate = mouseRotate;
    this.mousePitch = mousePitch;
  }
  /**
   * Enables the "drag to rotate" interaction.
   *
   * @example
   * map.dragRotate.enable();
   */


  _createClass(DragRotateHandler, [{
    key: "enable",
    value: function enable() {
      this.mouseRotate.enable();

      if (this.pitchWithRotate) {
        this.mousePitch.enable();
      }
    }
    /**
     * Disables the "drag to rotate" interaction.
     *
     * @example
     * map.dragRotate.disable();
     */

  }, {
    key: "disable",
    value: function disable() {
      this.mouseRotate.disable();
      this.mousePitch.disable();
    }
    /**
     * Returns a Boolean indicating whether the "drag to rotate" interaction is enabled.
     *
     * @returns {boolean} `true` if the "drag to rotate" interaction is enabled.
     */

  }, {
    key: "isEnabled",
    value: function isEnabled() {
      return this.mouseRotate.isEnabled() && (!this.pitchWithRotate || this.mousePitch.isEnabled());
    }
    /**
     * Returns a Boolean indicating whether the "drag to rotate" interaction is active, i.e. currently being used.
     *
     * @returns {boolean} `true` if the "drag to rotate" interaction is active.
     */

  }, {
    key: "isActive",
    value: function isActive() {
      return this.mouseRotate.isActive() || this.mousePitch.isActive();
    }
  }]);

  return DragRotateHandler;
}();

export { DragRotateHandler as default };